/*********************************************************************
 * Capítulo 3. Tutorial de desarrollo con OpenFL. Scroll Parallax.
 * Clase de Utilidad: Muestra los FPS empleando una fuente True Type. 
 * Programación Multimedia y Juegos
 * Autor: David Vallejo Fernández    david.vallejo@openflbook.com
 * Autor: Carlos González Morcillo   carlos.gonzalez@openflbook.com
 * Autor: David Frutos Talavera      david.frutos@openflbook.com
 *
 * You can redistribute and/or modify this file under the terms of the
 * GNU General Public License ad published by the Free Software
 * Foundation, either version 3 of the License, or (at your option)
 * and later version. See <http://www.gnu.org/licenses/>.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.  
 *********************************************************************/

package book.openfl.scrollParallax;

import book.openfl.scrollParallax.util.FpsLabel;
import book.openfl.scrollParallax.graphics.ParallaxManager;

import aze.display.TileLayer;
import aze.display.SparrowTilesheet;
import openfl.Assets;
import flash.display.Sprite;
import flash.events.Event;
import flash.Lib;
import flash.net.URLLoader;
import flash.net.URLRequest;
class Parallax extends Sprite {

  var fpstext:FpsLabel;                  // Muestra los FPS
  var parallaxManager:ParallaxManager;   // Manager del Efecto

  var layer:TileLayer;                   // Capa general
  var previousTime:Int;                  // Para calcular el delta
  var sheetData:String;
  // Constructor =====================================================
  public function new() {
    super(); haxe.Timer.delay(init, 150);
  }
	
  function init():Void {
    //createScene();
	#if js
	// workaround for binary data loading bug in html5
	var ul = new URLLoader(new URLRequest("img/bee.xml")); 
	ul.addEventListener(Event.COMPLETE, ul_complete);
	#else
	sheetData = Assets.getText("img/bee.xml");
	createScene();
	#end 

  }

  	function ul_complete(e)
	{
		sheetData = e.target.data;		

		
		createScene();	
	}
  // Creación de la Escena ===========================================
  function createScene():Void {
    //loadAssets();	
	var tilesheet:SparrowTilesheet = new SparrowTilesheet
	(Assets.getBitmapData("img/bee.png"), sheetData);
	layer = new TileLayer(tilesheet);
	addChild(layer.view);
    fpstext = new FpsLabel(20, 20, 0x606060, "font/cafeta.ttf");  
    addChild(fpstext);
    parallaxManager = new ParallaxManager(layer);
    parallaxManager.addSky("sky");
    parallaxManager.addBackgroundStrip("2cloud2", 10, 1.7, 300, -5);
    parallaxManager.addBackgroundStrip("2cloud1", 10, 2, 320, 10);
    parallaxManager.addBackgroundStrip("trees", 4, 1, 220, 40);
    parallaxManager.addBackgroundStrip("sheeps", 4, 1, 160, 140);
    parallaxManager.addBackgroundStrip("clouds", 4, 1.3, 280, 80);
    parallaxManager.addBackgroundStrip("flowers", 4, 1, 70, 300);
	addListeners();
    
  }    

  // Bucle Principal =================================================
  function mainLoop(eTime:Int):Void {
    fpstext.update();
    parallaxManager.update(stage.stageWidth, stage.stageHeight, eTime);
    layer.render();
  }

  // Listeners =======================================================
  function addListeners():Void {
    stage.addEventListener(Event.ENTER_FRAME, onEnterFrame);
	previousTime = Lib.getTimer();
  }
  
  function onEnterFrame(event:Event):Void {
    var now = Lib.getTimer();
    var elapsedTime = now - previousTime;
    previousTime = now;
    mainLoop(elapsedTime);
  }

  // Main ============================================================  
  static public function main() {
    Lib.current.addChild(new Parallax());
  }

}
